@readcircimage.pro

;----------------------------------------------------------
;  CALCULATE VISIBILITY VALUE AT GIVEN PROJECTED BASELINE
;              (VERSION FOR CIRCULAR IMAGES)
;
; This routine computes the visibility value of an image
; produced with the circular image option of RADICAL. This
; is a weird, but very useful way of making images of 2-D
; axisymmetric astrophysical objects with a huge radial
; span. Since the coordinates of the image are not X and Y
; but R and Phi, in which both R and Phi can be spaced in
; any way one likes. For e.g. disks with radial span of
; [0.1,1000]*AU the R coordinate of the image should then
; simply be logarithmically spaced to resolve the 0.1 AU
; as well as the 1000 AU structure. Since this allows you
; to make sure that the image contains ALL structure of
; the object at all scales, the 'near' and 'far' return
; values of the visibility.pro routine (on which this
; routine is based) are no longer useful, and therefore
; they are removed. 
;
; ARGUMENTS:
;   image           The image structure (see readimagecirc.pro)
;   d_pc            The distance in parsec
;   lambda_micron   The wavelength in micron
;   baseline_meter  The projected baseline in meters
;   ori_deg         The projection angle in degrees. Zero
;                   means along the x axis (first index)
;                   and 90 means along the y axis.
;   beam            The primary beam RADIUS in arcsec
;   method          =1 : Use sampling method (fast, but can go wrong
;                        for very large k vector)
;                   =2 : Use analytic integral method (slower, but
;                        much more stable)
;                   NOTE: Must specify method!
;
; RETURNS:
;   visibility      The visibility value
;   phase           The phase (for use for differential
;                   phase)
;   totflux         The total flux within the beam
;   corflux         The correlated flux within the beam
;
; NOTE:
;   Bug fixed (14-01-04): Now do both cos and sin and
;      compute the sqrt(cos^2+sin^2).
;   Large K problem fixed (08-02-07): Now each pixel-cell
;      is integrated over if method=2 is chosen. This is 
;      more stable for large k vector or large r vector.
;      In the original method this could lead to sampling
;      errors.
; NOTE: Roy's bug-fixes are built in. (08.02.07)
;----------------------------------------------------------
function visibility_circ,image,$
                    d_pc,$
                    lambda_micron,$
                    baseline_meter,$
                    ori_deg,$
                    beam=beam,$
                    phase=phase,$
                    totflux=totflux,$
                    corflux=corflux,$
                    method=method
;
; Check
;
if not keyword_set(method) then begin
    print,'ERROR: Must specify method!'
    stop
endif
;
; Get sizes
;
nr   = n_elements(image.r)
nf   = n_elements(image.phi)
sz   = size(image.image)
if sz(1) ne nr+1 then begin
   print,'ERROR in visibility_circ: the circular image'
   print,'      must have nr+1 radial pixels (the ir=0'
   print,'      point being the central point source)'
   stop
endif
if sz(2) ne nf then begin
   print,'ERROR in visibility_circ: the circular image'
   print,'      must have nf pixels in phi'
   stop
endif
;
; Get the coordinates
;
r    = image.r
ri   = image.ri
phi  = image.phi
phii = image.phii
;
; Convert to CGS
;
b    = baseline_meter * 1d2
d    = d_pc * 3.08572d18
lmb  = lambda_micron * 1d-4
ori  = ori_deg / 57.2957795131d0
;
; Find |k|
;
kk   = 2*!dpi*b/(lmb*d)
;
; Calculate the beam in units of cm
;
if n_elements(beam) ne 0 then begin
   beamcm=d_pc*1.496d13*beam
endif else begin
   beamcm=1d99
endelse
aprt = r lt beamcm
;
; Now calculate the visibility cosine and sine values
;
; Reset
;
sumtot=0.d0
sumftc=0.d0
sumfts=0.d0
;
; First the nebula
;
for ir=0,nr-1 do begin
   for iphi=0,nf-1 do begin
       case method of 
           1: begin
               srf    = 0.5*(phii[iphi+1]-phii[iphi])*(ri[ir+1]^2-ri[ir]^2)/$
                        (d_pc^2*9.5216679184d36)
               factc  = cos(cos(ori-phi[iphi])*kk*r[ir])
               facts  = sin(cos(ori-phi[iphi])*kk*r[ir])
               sumtot = sumtot + image.image[ir+1,iphi]*srf*aprt[ir]
               sumftc = sumftc + factc*image.image[ir+1,iphi]*srf*aprt[ir]
               sumfts = sumfts + facts*image.image[ir+1,iphi]*srf*aprt[ir]
           end
           2: begin
               k=kk
               srf   = 0.5d0*(ri[ir+1]+ri[ir])*(phii[iphi+1]-phii[iphi])* $
                        (ri[ir+1]-ri[ir])
               dstf   = (d_pc^2*9.5216679184d36)
               if abs(cos(phii[iphi+1]-ori)-cos(phii[iphi]-ori)) $
                 gt 1d-6 then begin
                   dum_cos = -(1.d0/(k*sin(0.5*$
                            (phii[iphi]+phii[iphi+1])-ori))) * $
                     ( (1.d0/(k*cos(phii[iphi+1]-ori))) * $
                       ( cos(k*ri[ir]*cos(phii[iphi+1]-ori)) $
                         -cos(k*ri[ir+1]*cos(phii[iphi+1]-ori)) ) - $
                       (1.d0/(k*cos(phii[iphi]-ori))) * $
                       ( cos(k*ri[ir]*cos(phii[iphi]-ori)) $
                         -cos(k*ri[ir+1]*cos(phii[iphi]-ori)) ) )
                   dum_sin = -(1.d0/(k*sin(0.5*$
                            (phii[iphi]+phii[iphi+1])-ori))) * $
                     ( (1.d0/(k*cos(phii[iphi+1]-ori))) * $
                       ( sin(k*ri[ir]*cos(phii[iphi+1]-ori)) $
                         -sin(k*ri[ir+1]*cos(phii[iphi+1]-ori)) ) - $
                       (1.d0/(k*cos(phii[iphi]-ori))) * $
                       ( sin(k*ri[ir]*cos(phii[iphi]-ori)) $
                         -sin(k*ri[ir+1]*cos(phii[iphi]-ori)) ) )
               endif else begin
                   dum_cos = srf*cos(k*ri[ir]*cos(phii[iphi]-ori))
                   dum_sin = srf*sin(k*ri[ir]*cos(phii[iphi]-ori))
               endelse
               sumtot = sumtot + image.image[ir+1,iphi]*srf*aprt[ir]/dstf
               sumftc = sumftc + dum_cos*image.image[ir+1,iphi]*aprt[ir]/dstf
               sumfts = sumfts + dum_sin*image.image[ir+1,iphi]*aprt[ir]/dstf
           end
           else: begin
               print,'ERROR: Method unknown!'
               stop
           end
       endcase
   endfor
endfor
;
; Then add the central point source
;
for iphi=0,nf-1 do begin
   srf    = 0.5*(phii[iphi+1]-phii[iphi])*ri[0]^2/(d_pc^2*9.5216679184d36)
   fact   = 1.d0
   sumtot = sumtot + image.image[0,iphi]*srf
   sumftc = sumftc + fact*image.image[0,iphi]*srf
endfor
;
; Find the ratio 
;
visc = sumftc/sumtot
viss = sumfts/sumtot
;
; The absolute value of the visibility vector is the visibility value
;
vis  = sqrt(visc^2+viss^2)
;
; The atan of the ratio is the phase
; MODIFIED: 14.11.06: added 1d-99
;
phase = atan(viss/(visc+1d-99))
;
; The total flux and the correlated flux
;
totflux = sumtot
corflux = sqrt(sumftc^2+sumfts^2)
;
; Return the value
;
return,vis
end



;----------------------------------------------------------
;         CALCULATE TOTAL AND CORRELATED SPECTRUM
; 
; This subroutine calls RADICAL in the appropriate way
; to compute the total flux and the correlated flux within
; a given beam (within a wavelength range if specified).
;
; ARGUMENTS:
;   d_pc            The distance in parsec
;   baseline_meter  The projected baseline in meters
;   incl_deg        The inclination angle in degrees. Zero
;                   means pole-on.
;   ori_deg         The projection angle in degrees. Zero
;                   means along the x axis (first index)
;                   and 90 means along the y axis.
;   beam            The primary beam in arcsec.
;                   If one number, then this is the beam
;                   for all wavelengths. If an array and
;                   if blam is a same-size array, then 
;                   beam(blam) is a wavelength-dependent
;                   beam size. Beware that for the lambdas 
;                   outside your blam range the beam is 
;                   assumed to be 0.
;                   
;   blam            See text for `beam'
;   lamrange        The range within which to make the
;                   total and correlated spectra.
;
;
;----------------------------------------------------------
function corrspec,d_pc,baseline_meter,incl_deg,ori_deg,$
                  beam=beam,blam=blam,lamrange=lamrange,$
                  write=write
;
; First read the frequency.inp file
;
openr,1,'frequency.inp'
nf=0
readf,1,nf
freq=dblarr(nf)
readf,1,freq
close,1
lambda=2.9979d14/freq
;
; Now select the wavelengths
;
if keyword_set(lamrange) then begin
   ii = where((lambda ge lamrange[0]) and (lambda le lamrange[1]))
   if ii[0] lt 0 then begin
      return,-1
   endif
   lambda = lambda[ii]
   freq = freq[ii]
endif else begin
   ii = floor(findgen(nf))
endelse
ni=n_elements(ii)
if ii[0] lt 0 then begin
   return,-1
endif
;
; Now do a loop over the wavelength range
;
spectot = dblarr(ni)
speccor = dblarr(ni)
visib   = dblarr(ni)
for i0=0,ni-1 do begin
   inu = ii[i0]
   ;;
   ;; Make the circular image
   ;;
   makecircimage,incl=incl_deg,ifreq=ii[i0]+1
   ;;
   ;; Read the circular image
   ;;
   a   = readcircimage()
   ;;
   ;; Find the beam
   ;;
   if n_elements(beam) eq 0 then begin
      bm = 1d99
   endif else begin
      if n_elements(beam) eq 1 then begin
         bm = beam
      endif else begin
         if((lambda[i0] lt min(blam)) or (lambda[i0] gt max(blam))) then begin
            bm = 0.d0
         endif else begin
            bm = interpol(beam,blam,lambda[i0])
         endelse
      endelse
   endelse
   ;;
   ;; Compute the flux and correlated flux
   ;;
   if bm gt 0.d0 then begin
      vis = visibility_circ(a,d_pc,lambda[i0],baseline_meter,ori_deg,$
                            beam=bm,totflux=totflux,corflux=corflux)
   endif else begin
      vis     = 0.d0
      totflux = 0.d0
      corflux = 0.d0
   endelse
   spectot[i0] = totflux
   speccor[i0] = corflux
   visib[i0]   = corflux/totflux
endfor
;
; If write then write the two spectra to the usual files
;
if keyword_set(write) then begin
   openw,1,'spectrum_beamtot.dat'
   printf,1,ni
   for i0=0,ni-1 do printf,1,2.9979d14/lambda[i0],spectot[i0]*d_pc^2,format='(2(E13.6,1X))'
   close,1
   openw,1,'spectrum_beamcor.dat'
   printf,1,ni
   for i0=0,ni-1 do printf,1,2.9979d14/lambda[i0],speccor[i0]*d_pc^2,format='(2(E13.6,1X))'
   close,1
endif
;
return,{lambda:lambda,spectot:spectot,speccor:speccor,visib:visib}
end
