#include "main.h"
c     -----------------------------------------------------------------
c                  EXTERNAL RADMC MODULE FOR PAH EMISSION 
c
c     This program computes PAH emission using the absorption/emission
c     cross-sections computed by the program of Ruud Visser. The 
c     emission is here computed using the method of Leger & Puget 1984.
c     The formulae are from the papers of Siebenmorgen & Kruegel 1992
c     and Siebenmorgen, Kruegel & Mathis 1992. The method can only
c     compute single-photon events.
c
c     It is used as an external module to RADMC (will later be included
c     into RADMC itself). First do one run with RADMC, then one run 
c     with this PAH code, then another run with RADMC (the latter is
c     not so very important; it includes the PAH emission as heating
c     source for the thermal grains).
c     -----------------------------------------------------------------
      program pah_radmc
      implicit none
      integer iformat,itemp,inu,inu2,idum,nr,nt,imirt,ir,it,ispec
      doubleprecision temp0,temp1
      doubleprecision radfield(FRSIZE_FREQ),rho_q
      doubleprecision emissivity(FRSIZE_FREQ)
      doubleprecision destrtime
      logical fex
c     
#include "common_dust.h"
#include "common_grid.h"
#include "common_pah_radmc.h"
c
      write(*,*) 'One-photon version of PAH code'
c
c     Defaults
c
      temp0     = 0.1
      temp1     = 1d6
      ntempgrid = 1000
      if(ntempgrid.gt.QUANTUM_NTEMP) then 
          write(*,*) 'ERROR pah_onephot: ntempgrid out of range'
          stop
      endif
c
c     Read all
c
      write(*,*) 'Read data'
      call read_all()
c
c     Define imirt and nr and nt
c
      nr    = irsi_frsizex
#     ifdef MIRROR_THETA
        nt    = (irsi_frsizey+1)/2
        imirt = 1
#     else
c        nt = irsi_frsizey
c        imirt = 0
      stop
#     endif
c
c     Make a grid of temperatures
c
      do itemp=1,ntempgrid
          tempgrid(itemp) = temp0 * (temp1/temp0)**
     %                      ((itemp-1.d0)/(ntempgrid-1.d0))
      enddo
c
c     Clear the emissivity array
c
      do ir=1,nr
          do it=1,nt
              do inu=1,freq_nr
                  emisquant(inu,it,ir) = 0.d0
              enddo
          enddo
      enddo
c
c     Do a loop over all dust species, check if they are PAH/small
c     carbonateous grains, and if so, then compute the whole stuff
c
      do ispec=1,dust_nr_species
c
c         Check if quantum heated
c
          if(dust_quantum(ispec).ne.0) then
c
c             Yes, this is a quantum heated grain
c
              write(*,*) 'Doing ispec=',ispec
              write(*,*) '  Computing response matrix...'
c
c             Set up the heat content and peak temperatures
c
              call compute_peak_temperatures(ispec)
c
c             Compute cooling time curve
c
              call pah_cooling(ispec)
c
c             Compute the emission of PAH molecules as they cool down
c
              call emis_cool(ispec)
c
c             Make response matrix
c
              call make_response_matrix(ispec)
c
c             Now make a loop over all of space
c
              write(*,*) '  Computing the local emissivity...'
              do ir=1,nr
                  do it=1,nt
c
c                     Copy mean intensity of exciting radiation 
c                     into local array
c
                      do inu=1,freq_nr
                          radfield(inu) = miquant(inu,it,ir)
                      enddo
c
c                     Copy density of quantum heated grain
c
                      rho_q = dust_rho(ispec,it,ir)
c
c                     Compute the emissivity
c
                      call compute_emissivity_pah(rho_q,
     %                               radfield,emissivity)
c
c                     Add emissivity to global array
c
                      do inu=1,freq_nr
                          emisquant(inu,it,ir) = emisquant(inu,it,ir) +
     %                                           emissivity(inu)
                      enddo
c
c                     Compute the destruction rate
c
                      call compute_destruction_pah(radfield,
     %                                             ispec,destrtime)
c
c                     Put this into global array
c
                      pahdes_time_store(ispec,it,ir) = destrtime
c
                  enddo
              enddo
          endif
      enddo
c
      write(*,*) 'Writing results'
c
c     Write the emisquant
c
      open(unit=1,file='quantsource.dat')
      write(1,*) freq_nr,nr,nt,1
      do inu=1,freq_nr
          do it=1,nt
              do ir=1,nr
                  write(1,*) emisquant(inu,it,ir)
              enddo
          enddo
          write(1,*) ' '
      enddo
      close(1)
c
c     Write peak temperatures to file
c
      open(unit=1,file='peaktemp.dat')
      write(1,*) freq_nr,dust_nr_species
      do ispec=1,dust_nr_species
          do inu=1,freq_nr
              write(1,*) freq_nu(inu),peak_temp_store(inu,ispec)
          enddo
          write(1,*) ' '
      enddo
      close(1)
c
c     Write the cooling curve to file
c
      open(unit=1,file='cooling.dat')
      write(1,*) ntempgrid,dust_nr_species
      do ispec=1,dust_nr_species
          do itemp=1,ntempgrid 
              write(1,*) tempgrid(itemp),cooltime_store(itemp,ispec)
          enddo
          write(1,*) ' '
      enddo
      close(1)
c
c     Write the life time for the PAH molecules
c
      open(unit=1,file='lifetime.dat')
      write(1,*) dust_nr_species,nr,nt
      do ispec=1,dust_nr_species
          do it=1,nt
              do ir=1,nr
                  write(1,*) pahdes_time_store(ispec,it,ir)
              enddo
          enddo
          write(1,*) ' '
      enddo
      close(1)
c
      end


c     --------------------------------------------------------------
c                     READ THE GRID AND SETUP
c     --------------------------------------------------------------
      subroutine read_all()
      implicit none
      doubleprecision dummy,frq,dum
      integer i,ir,it,nr,nt,ix,iy,itmax,k,j,imirt,inu
      integer iformat,ispec,nnf
      logical fex
c
#include "common_dust.h"
#include "common_grid.h"
#include "common_pah_radmc.h"
c#include "common_montecarlo.h"
c
c     Read the R grid
c     
      open(unit=3,file='radius.inp',status='old')
      read(3,*) irsi_frsizex
      if(irsi_frsizex.gt.FRSIZE_X) then
          write(*,*) 'ERROR: radius.inp has more radial gridpoints'
          write(*,*) '       than FRSIZE_X. '
          write(*,*) '       Recompile with larger FRSIZE_X'
          stop 13
      endif
      do ix=1,irsi_frsizex
          read(3,*) rsi_x_c(ix,1)
      enddo
      close(3)
c     
c     The boundaries
c
      rsi_x_c(0,1)  = rsi_x_c(1,1)**2 / rsi_x_c(2,1)
      rsi_x_c(-1,1) = rsi_x_c(0,1)**2 / rsi_x_c(1,1)
      rsi_x_c(irsi_frsizex+1,1) = rsi_x_c(irsi_frsizex,1)**2 
     %     / rsi_x_c(irsi_frsizex-1,1)
      rsi_x_c(irsi_frsizex+2,1) = rsi_x_c(irsi_frsizex+1,1)**2 
     %     / rsi_x_c(irsi_frsizex,1)
c
c     The cell interfaces
c
      do ix=0,irsi_frsizex+2
          rsi_x_i(ix,1)  = sqrt(rsi_x_c(ix,1)*rsi_x_c(ix-1,1))
      enddo
c
c     Read Theta grid from file, and recompute all other data. 
c     Beware of the theta mirroring option.
c     
      open(unit=3,file='theta.inp',status='old')
      read(3,*) irsi_frsizey,imirt
      itmax = irsi_frsizey
      do iy=1,itmax
          read(3,*) rsi_x_c(iy,2)
          if(rsi_x_c(iy,2).lt.0.d0) then
              write(*,*) 'ERROR: theta.inp has negative values.'
              stop 13                  
          endif
      enddo
      if(imirt.ne.0) then
          irsi_frsizey = 2*irsi_frsizey
      endif
      if(irsi_frsizey.gt.FRSIZE_Y) then
          write(*,*) 'ERROR: theta.inp has more Theta gridpoints'
          write(*,*) '       than FRSIZE_Y. '
          write(*,*) '       Recompile with larger FRSIZE_Y'
          stop 13
      endif
      if(imirt.ne.0) then
          do iy=1,itmax
              rsi_x_c(2*itmax+1-iy,2) = 
     %             3.14159265359d0 - rsi_x_c(iy,2)
          enddo      
      endif
      close(3)
c
c     Do boundaries right
c
      rsi_x_c(0,2)  = - rsi_x_c(1,2)
      rsi_x_c(-1,2) = - rsi_x_c(2,2)
      rsi_x_c(irsi_frsizey+1,2) = 2 * 3.1415926e0 
     %                  - rsi_x_c(irsi_frsizey,2)
      rsi_x_c(irsi_frsizey+2,2) = 2 * 3.1415926e0 
     %                  - rsi_x_c(irsi_frsizey-1,2)
c         
c     Cell interfaces
c         
      do iy=0,irsi_frsizey+2
          rsi_x_i(iy,2)  = 0.5d0 * ( rsi_x_c(iy-1,2) + 
     %         rsi_x_c(iy,2) )
      enddo
c
c     Read the frequency
c     
      open(unit=1,file='frequency.inp',status='old')
      read(1,*) freq_nr
      if(freq_nr.gt.FRSIZE_FREQ) then
          write(*,*) 'ERROR: frequency.inp has more gridpoints'
          write(*,*) '       than FRSIZE_FREQ. '
          write(*,*) '       Recompile with larger FRSIZE_FREQ'
          stop 13
      endif          
      do inu=1,freq_nr
          read(1,*) freq_nu(inu)
      enddo
      close(1)
c
c     Read the dust opacities
c
      call read_dustdata()
c
c     Read the dust density
c
      open(unit=1,file='dustdens.inp',status='old')
      read(1,*) k,nr,nt,j
      if((nr.ne.irsi_frsizex).or.(nt.ne.irsi_frsizey/2)) stop 721
      if(k.ne.dust_nr_species) then
          write(*,*) 'ERROR: In the dustopac.inp file the number'
          write(*,*) '  of dust species is different from that '
          write(*,*) '  specified in dustdens.inp'
          stop 13
      endif
      do ispec=1,dust_nr_species
          do ir=1,nr
              do it=1,nt
                  read(1,*) dum
                  if(dum.gt.1d-90) then
                      dust_rho(ispec,it,ir) = dum
                  else
                      dust_rho(ispec,it,ir) = 1d-90
                  endif
              enddo
          enddo
      enddo
      close(1)
c
c     Read the mean intensity
c
      open(unit=1,file='meanint_quantum.dat',status='old')
      read(1,*) k,nr,nt,j
      if((nr.ne.irsi_frsizex).or.(nt.ne.irsi_frsizey/2)) stop 721
      if(k.ne.freq_nr) then
          write(*,*) 'ERROR: The number of frequency-points in'
          write(*,*) '  meanint_quantum.dat does not correspond '
          write(*,*) '  to that in frequency.inp'
          stop 13
      endif
      do inu=1,freq_nr
          do it=1,nt
              do ir=1,nr
                  read(1,*) dum
                  miquant(inu,it,ir) = dum
              enddo
          enddo
      enddo
      close(1)
c
      end



c     -----------------------------------------------------------------
c                   COMPUTE THE EMISSION FROM PAH
c
c     This routine computes the emissivit of the PAH molecules in a
c     radiation field given by the mean intensity J_nu
c     (erg/s/cm^2/Hz/sterradian).  It is assumed that all of this
c     radiation is quantum-heating radiation according to a 1-photon
c     heating process. This means that before the next heating-photon
c     arrives, the PAH has cooled down to the lowest temperature of the
c     temperature grid. The computed emissivity is the emissivity per
c     gram of PAH. So this must be multiplied with rho_PAH (gram/cm^3)
c     to obtain the actual emissivity j_nu (erg/s/cm^3/Hz/sterradian).
c
c     INPUT:
c       rho_pah         The density (gram/cm^3) of PAH dust
c       Jnu_excite      The J_nu (mean intensity) of the exciting 
c                       photons
c
c     RETURNS:
c       emis_pah        The j_nu (emissivity of the PAHs)
c
c     -----------------------------------------------------------------
      subroutine compute_emissivity_pah(rho_pah,Jnu_excite,emis_pah)
      implicit none
c
#include "common_dust.h"
#include "common_grid.h"
#include "common_pah_radmc.h"
c
      integer inui,inuo
      doubleprecision Jnu_excite(FRSIZE_FREQ),emis_pah(FRSIZE_FREQ)
      doubleprecision dnu(FRSIZE_FREQ),rho_pah
c
c     First reset the emission
c
      do inuo=1,freq_nr
          emis_pah(inuo) = 0.d0
      enddo
c
c     Make dnu
c
      dnu(1)       = 0.5 * ( freq_nu(2) - freq_nu(1) )
      dnu(freq_nr) = 0.5 * ( freq_nu(freq_nr) - freq_nu(freq_nr-1) )
      do inui=2,freq_nr-1
          dnu(inui) = 0.5 * ( freq_nu(inui+1) - freq_nu(inui-1) )
      enddo
c
c     Do a loop over all the input frequencies
c
      do inui=1,freq_nr
          do inuo=1,freq_nr
              emis_pah(inuo) = emis_pah(inuo) + rho_pah * 
     %              Jnu_excite(inui) * dnu(inui) * response(inuo,inui)
          enddo
      enddo
c
      end



c     -----------------------------------------------------------------
c                   COMPUTE THE DESTRUCTION OF PAHs
c
c     This routine computes the 
c
c     INPUT:
c       Jnu_excite      The J_nu (mean intensity) of the exciting 
c                       photons
c
c     RETURNS:
c       destrtime       E-folding life-time of the PAHs (the grains
c                       that are not PAH get life-time 0.d0).
c
c     -----------------------------------------------------------------
      subroutine compute_destruction_pah(Jnu_excite,ispec,destrtime)
      implicit none
c
#include "common_dust.h"
#include "common_grid.h"
#include "common_pah_radmc.h"
c
      integer ispec,inu
      doubleprecision Jnu_excite(FRSIZE_FREQ)
      doubleprecision destrtime,dum,mp
      doubleprecision dnu(FRSIZE_FREQ)
      parameter(mp=1.6726d-24)
c
c     Make dnu
c
      dnu(1)       = 0.5 * ( freq_nu(2) - freq_nu(1) )
      dnu(freq_nr) = 0.5 * ( freq_nu(freq_nr) - freq_nu(freq_nr-1) )
      do inu=2,freq_nr-1
          dnu(inu) = 0.5 * ( freq_nu(inu+1) - freq_nu(inu-1) )
      enddo
c
c     Do a loop over all the input frequencies
c
      if(dust_quantum(ispec).ne.0) then
          dum = 0.d0
          do inu=1,freq_nr
              dum = dum + Jnu_excite(inu) * 
     %              dnu(inu) * pahdes_rate(inu) *
     %              12 * mp * dust_n_catom(ispec)
          enddo
          if(dum.gt.0.d0) then
              destrtime = 1.d0 / dum
          else
              destrtime = 1d30
          endif
      else
          destrtime = 0.d0
      endif
c
      end



c     -----------------------------------------------------------------
c        COMPUTE RESPONSE MATRIX (=EMISSIVITY AFO INPUT SPECTRUM)
c
c     This routine computes an emissivity j_nu as a function of input
c     frequency (i.e. assuming the exciting photon has this input
c     frequency). Once this matrix is made, the actual output from 
c     some input spectrum is simply an integral of spectra.
c     -----------------------------------------------------------------
      subroutine make_response_matrix(ispec)
      implicit none
c
#include "common_dust.h"
#include "common_grid.h"
#include "common_pah_radmc.h"
c
      integer inui,inuo,ispec,itemp
      doubleprecision pi,hh,kappa(FRSIZE_FREQ),find_dust_kappa
      doubleprecision nphot,grex,eps,flux,corrfact,mp
      parameter(pi=3.1415926535897932385d0) 
      parameter(hh=6.6262d-27)
      parameter(mp=1.6726d-24)
c
c     Get the kappa
c
      do inui=1,freq_nr
          kappa(inui) = find_dust_kappa(inui,1,ispec,10.,1,0)
      enddo
c
c     Now do a loop over frequency of exciting photons
c
      do inui=1,freq_nr
c
c         Find out how many photons are absorbed per second
c         by the PAHs in a radiation field at this frequency
c         of 1 erg/s/cm^2/Hz/ster (still has to be multiplied
c         by rho_pah and dnu)
c
          nphot = 4 * pi * kappa(inui) / ( hh * freq_nu(inui) )
c
c         Convert this in the amount of gram / cm^3 / s of
c         PAH molecules are excited
c
          grex  = nphot * 12 * mp * dust_n_catom(ispec)
c
c         The peak temperature
c         
          itemp = peak_itemp(inui)
          eps   = peak_eps(inui)
c          if(itemp.ge.ntempgrid) stop 7801
c
c         If the peak temperature does not exceed evaporation temperature /
c         dissociation energy, then include this to the emission
c
          if((itemp.gt.0).and.(itemp.lt.ntempgrid)) then
c
c             Apparently the photon has not destroyed the grain,
c             so make the spectrum
c
              do inuo=1,freq_nr
                  response(inuo,inui) = grex * 
     %                 ( (1-eps) * emistemp(inuo,itemp) +
     %                 eps * emistemp(inuo,itemp+1) )
              enddo
c
c             Check the flux conservation
c             
              flux = 0.d0
              do inuo=2,freq_nr
                  flux = flux + 0.5 * 
     %                 ( response(inuo,inui) + 
     %                   response(inuo-1,inui) ) * 
     %                 ( freq_nu(inuo) - freq_nu(inuo-1) )
              enddo
              flux = flux * 4 * pi ! Because emis = q/sterradian )
              corrfact = nphot*hh*freq_nu(inui)/flux
              if(abs(corrfact-1.d0).gt.0.1) then
                  write(*,*) 'PROBLEM: Flux conservation went wrong'
                  write(*,*) '   with a factor ',corrfact
                  stop
              endif
c             
c             Make the small correction for the flux conservation
c             
              do inuo=1,freq_nr
                  response(inuo,inui) = response(inuo,inui) * corrfact
              enddo
c
c             This PAH is not destroyed
c
              pahdes_rate(inui) = 0.d0
c
          else
c
c             The exciting photon has destroyed the PAH molecule.
c             This is lost energy, and it is unclear how/if this
c             energy is re-emitted in some form of radiation.
c
              do inuo=1,freq_nr
                  response(inuo,inui) = 0.d0
              enddo
c
c             Nphot is how many PAH molecules of this type are being
c             destroyed per second per gram PAH for a radiation
c             field at this frequency of J=1.
c
              pahdes_rate(inui) = nphot
c
          endif
c
      enddo
c
      end



c     -----------------------------------------------------------------
c                COMPUTE PEAK TEMPERATURE FOR EXCITED PAH
c
c     This subroutine computes the peak temperature obtained by a
c     single photon with frequency nu
c     -----------------------------------------------------------------
      subroutine compute_peak_temperatures(ispec)
      implicit none
      doubleprecision peak_temperature,hh,cv_pah,energy
      integer itemp,ispec,inu
      parameter(hh=6.6262d-27)
c
#include "common_dust.h"
#include "common_grid.h"
#include "common_pah_radmc.h"
c
c     Compute a table for the heat content
c
c     First one is zero
c
      heatcontent(1) = 0.d0
c
c     Now compute the rest
c     
      do itemp=2,ntempgrid
          heatcontent(itemp) = heatcontent(itemp-1) + 
     %          dust_n_catom(ispec) * cv_pah(tempgrid(itemp)) * 
     %          (tempgrid(itemp)-tempgrid(itemp-1))
      enddo
c
c     Now do loop over frequency, and compute the peak temperature
c     at every frequency (when a photon of that frequency hits
c     a PAH).
c
      do inu=1,freq_nr
c
c         Compute the energy of a single photon
c
          energy = hh * freq_nu(inu)
c
c         Find the index of the heat content corresponding
c         to that energy. This gives the peak temperature
c
          if(energy.lt.heatcontent(ntempgrid-1)) then
c
c             Find the index of the temperature
c
              call hunt(heatcontent,ntempgrid,energy,itemp)
c
c             Find the actual temperature through linear interpolation
c
              peak_itemp(inu) = itemp
              peak_eps(inu)   = ( energy - heatcontent(itemp) ) / 
     %            ( heatcontent(itemp+1) - heatcontent(itemp) )
              if((peak_eps(inu).lt.0.d0).or.
     %           (peak_eps(inu).gt.1.d0)) then
                  stop 82701
              endif
              peak_temp(inu)  = 
     %               (1.d0-peak_eps(inu)) * tempgrid(itemp) + 
     %                      peak_eps(inu) * tempgrid(itemp+1)
c
c             If this temperature exceeds the evaporation temperature
c             then put peak_itemp to ntempgrid as a signal
c
              if((peak_temp(inu).gt.dust_tmax(ispec)).and.
     %           (dust_tmax(ispec).ne.0.d0)) then
                  peak_itemp(inu) = ntempgrid
              endif
          else
c              write(*,*) 'ERROR: Temperature grid has not high'
c              write(*,*) '       enough temperature to accomodate'
c              write(*,*) '       photons of highest frequency...'
c              stop
c
c             If the temperature is larger than the temperature
c             grid, then obviously it must have been evaporated.
c
              peak_itemp(inu) = ntempgrid
          endif
      enddo
c
c     For user-information, store the peak temperature
c
      do inu=1,freq_nr
          peak_temp_store(inu,ispec) = peak_temp(inu)
      enddo
c
      end


c     -----------------------------------------------------------------
c               COMPUTE EMITTED SPECTRUM DURING COOLING TIME
c
c     This routine computes the time-integrated emission over the
c     entire cooling time of the 1 gram of excited PAH molecules, when
c     the cooling starts at the specified temperature. Units
c     are erg/Hz/ster/gram. 
c     -----------------------------------------------------------------
      subroutine emis_cool(ispec)
      implicit none
c
#include "common_dust.h"
#include "common_grid.h"
#include "common_pah_radmc.h"
c
      integer itemp,inu,ispec
      doubleprecision temp,bplanck,kappa(FRSIZE_FREQ)
      doubleprecision find_dust_kappa
c
c     Get the kappa
c
      do inu=1,freq_nr
          kappa(inu) = find_dust_kappa(inu,1,ispec,10.,1,0)
      enddo
c
c     Emission from lowest temperature by definition 0
c
      do inu=1,freq_nr
          emistemp(inu,1) = 0.d0
      enddo
c
c     Emission emitted during cooling in each of the intervals
c
      do itemp=2,ntempgrid
c
c         Average temperature
c
          temp = 0.5d0 * ( tempgrid(itemp) + tempgrid(itemp-1) )
c
c         Compute emission for 1 gram of PAHs for this interval
c
          do inu=1,freq_nr
              emistemp(inu,itemp) = emistemp(inu,itemp-1) +
     %              bplanck(temp,freq_nu(inu)) * kappa(inu) *
     %              ( cooltime(itemp-1) - cooltime(itemp) )
          enddo
      enddo
c
      end


c     -----------------------------------------------------------------
c                    COMPUTE THE COOLING TIME LINE
c
c     This routine returns the time the PAH reaches each of the 
c     temperatures of the given temperature grid, starting from the
c     highest temperature (i.e. the last temperature in the array),
c     and cooling down to the lowest.
c     For this, the routine uses dust opacity nr ispec.
c     The method used here is from Leger & Puget (1984), see also
c     Siebenmorgen, Kruegel & Mathis (1992) page 6 (=506).
c     -----------------------------------------------------------------
      subroutine pah_cooling(ispec)
      implicit none
c
      integer ispec
      doubleprecision temp,dtime,cv_pah
      doubleprecision kappa(FRSIZE_FREQ),dum(FRSIZE_FREQ)
      doubleprecision find_dust_kappa,bplanck,integrate,pi,mp
      parameter(pi=3.1415926535897932385d0)
      parameter(mp=1.6726d-24)
c     
      integer itemp,inu
c
#include "common_dust.h"
#include "common_grid.h"
#include "common_pah_radmc.h"
c
c     Tests
c
      if((ispec.lt.1).or.(ispec.gt.dust_nr_species)) then
          write(*,*) 'ERROR pah_cooling(): ispec out of range'
          stop
      endif
      if(tempgrid(ntempgrid).le.tempgrid(1)) then
          write(*,*) 'Tempeature grid must be in ascending order'
          stop
      endif
c
c     Get the kappa
c
      do inu=1,freq_nr
          kappa(inu) = find_dust_kappa(inu,1,ispec,10.,1,0)
      enddo
c
c     Time at max temp = 0
c     
      cooltime(ntempgrid) = 0.d0
c
c     Now let's cool!
c
      do itemp=ntempgrid-1,1,-1 
c
c         Get average temperature
c
          temp = 0.5d0 * ( tempgrid(itemp+1) + tempgrid(itemp) )
c
c         Compute the specific heat c_V(T) in erg/gram/K
c         The 12 is because of the weight of a C-atom
c
          cv(itemp) = cv_pah(temp) / ( 12*mp ) 
c
c         Compute cooling rate per gram of dust
c
          do inu=1,freq_nr
              dum(inu) = bplanck(temp,freq_nu(inu)) * kappa(inu)
          enddo
          qcool(itemp) = 4 * pi * integrate(freq_nr,freq_nu,dum)
c
c         Compute the time it takes to cool...
c
          dtime = cv(itemp) * 
     %            ( tempgrid(itemp+1) - tempgrid(itemp) ) / 
     %              qcool(itemp)
c
c         Compute the new time
c
          cooltime(itemp) = cooltime(itemp+1) + dtime 
      enddo
c
c     For user-information, store the cooling time
c
      do itemp=1,ntempgrid
          cooltime_store(itemp,ispec) = cooltime(itemp)
      enddo
c
      end


c     -----------------------------------------------------------------
c                SPECIFIC HEAT FOR PAH MOLECULE
c
c     This is the specific heat per C-atom of a PAH molecule in
c     erg/atom/K.
c     -----------------------------------------------------------------
      function cv_pah(temp)
      implicit none
      doubleprecision cv_pah,temp,enthalpy_pah
      doubleprecision eps
      parameter(eps=1d-6)
c
      cv_pah = ( enthalpy_pah(temp*(1.d0+eps))-
     %           enthalpy_pah(temp*(1.d0)) ) / 
     %          (temp*eps)
c      write(*,*) cv_pah
c
      return
      end


c     -----------------------------------------------------------------
c                      ENTHALPY FOR PAH MOLECULES
c
c     According to Siebenmorgen & Kruegel (1992) the enthalpy of 
c     PAH molecules in erg/atom is (formula by Chase et al 1985):
c     -----------------------------------------------------------------
      function enthalpy_pah(temp)
      implicit none
      doubleprecision temp,enthalpy_pah
c
c     The formula of Chase
c
      enthalpy_pah = 4.15D-22 * (temp**3.3) / 
     %               ( 1.D0 + 6.51D-3*temp + 1.5D-6*temp**2 +
     %                 8.3D-7*(temp**2.3) )
c
      return
      end


c     --------------------------------------------------------------
c                THE BLACKBODY PLANCK FUNCTION B_nu(T)
c
c     This function computes the Blackbody function 
c
c                    2 h nu^3 / c^2
c        B_nu(T)  = -------------------    [ erg / cm^2 s ster Hz ]
c                   exp(-h nu / kT) - 1
c
c     ARGUMENTS:
c        nu    [Hz]            = Frequency
c        temp  [K]             = Electron temperature
c     --------------------------------------------------------------
      function bplanck(temp,nu)
      implicit none
      doubleprecision temp
      doubleprecision nu
      doubleprecision bplanck
c
      if(temp.eq.0.d0) then 
          bplanck = 0.d0
          return
      endif
c
      bplanck = 1.47455d-47 * nu * nu * nu / 
     %      (exp(4.7989d-11 * nu / temp)-1.d0) + 1.d-290
c
      return
      end

c     --------------------------------------------------------------
c                   A SIMPLE INTEGRATION ROUTINE
c     --------------------------------------------------------------
      function integrate(n,x,f)
      implicit none
      integer n,i
      doubleprecision x(n),f(n),integrate,int
      int=0.d0
      do i=2,n
          int=int+0.5d0*(f(i)+f(i-1))*(x(i)-x(i-1))
      enddo
      if(x(n).gt.x(1)) then
          integrate = int
      else
          integrate = -int
      endif
      return
      end


c     --------------------------------------------------------------
c                      NUMERICAL RECIPES ROUTINE
c     --------------------------------------------------------------
      SUBROUTINE hunt(xx,n,x,jlo)
      INTEGER jlo,n
      doubleprecision x,xx(n)
      INTEGER inc,jhi,jm
      LOGICAL ascnd
      ascnd=xx(n).gt.xx(1)
      if(jlo.le.0.or.jlo.gt.n)then
        jlo=0
        jhi=n+1
        goto 3
      endif
      inc=1
      if(x.ge.xx(jlo).eqv.ascnd)then
1       jhi=jlo+inc
        if(jhi.gt.n)then
          jhi=n+1
        else if(x.ge.xx(jhi).eqv.ascnd)then
          jlo=jhi
          inc=inc+inc
          goto 1
        endif
      else
        jhi=jlo
2       jlo=jhi-inc
        if(jlo.lt.1)then
          jlo=0
        else if(x.lt.xx(jlo).eqv.ascnd)then
          jhi=jlo
          inc=inc+inc
          goto 2
        endif
      endif
3     if(jhi-jlo.eq.1)return
      jm=(jhi+jlo)/2
      if(x.gt.xx(jm).eqv.ascnd)then
        jlo=jm
      else
        jhi=jm
      endif
      goto 3
      END
C  (C) Copr. 1986-92 Numerical Recipes Software =v1.9"217..

c     --------------------------------------------------------------
c                      MAKE INDEXED FILENAME
c
c     This routine makes filenames of the type dusttemp_4.dat etc.
c     --------------------------------------------------------------
      subroutine make_indexed_filename(base,index,ext,filename)
      implicit none
      character*80 base
      integer index
      character*80 ext
      character*80 filename
      character*12 ch
c     
      if((index.lt.0).or.(index.ge.1000)) then
          write(*,*) 'ERROR in make_indexed_filename()'
          stop 729
      endif
      if(index.lt.10) then
          write(ch,11) index
 11       format(I1)
      elseif(index.lt.100) then
          write(ch,12) index
 12       format(I2)
      elseif(index.lt.1000) then
          write(ch,13) index
 13       format(I3)
      endif
      filename = base(1:len_trim(base))//
     %           ch(1:len_trim(ch))//
     %           ext(1:len_trim(ext))
c
      end
