c     =============================================================
c                2-D SHORT CHARACTERISTIC TRANSFER ALGORITHM
c                 FOR POLAR COORDINATES AND AXIAL SYMMETRY
c
c        ( MODULE: FIND I,S,ALPHA BETWEEN GRIDPOINTS BY INTERPOLATION ) 
c
c                            Leiden, Aug 1998
c                             C.P. Dullemond   
c
c     =============================================================


#define INTERPOL_F_VERSION "Trans_05oct98"
#include "main.h"
#define SPL_IOFF 3
#define FRSIZE_PHI_SPLINE (FRSIZE_PHI+2*SPL_IOFF+1)

c#ifndef NOTRANS



c     --------------------------------------------------------------
c                MAKE THE INDEX ARRAYS FOR INTENSITY DATA
c
c     When mirror symmetries are imposed, the indexation of the 
c     intensity array goes rather different from normal. In order 
c     not to make the program unreadable and slow, we set up two
c     re-indexation arrays. One for iphi and one for itheta.
c
c     NOTE: The iangset argument is actually irrelevant. It could just
c           as well be 1. Note that the angle-grid (mu,phi) is (and
c           must be) the same at all ir! The reason why I have
c           a separate storage for the angle-grid for each iangset is
c           mainly for historic reason, and for a possible future
c           option of alternating mu-gridding. 
c     --------------------------------------------------------------
      subroutine make_index(iangset)
      implicit none
      integer iangset
c
#include "common_grid.h"
c
      integer it,ip,iphihalf,iphiquadr
      integer imu,ii
      doubleprecision drdivr,muu,mu1,mu2
c
      do it=-4,irsi_frsizey+4
          ridx_it(it) = it
          if(ridx_it(it).lt.1) then
              ridx_it(it) = 1 - ridx_it(it)
          endif
          if(ridx_it(it).gt.irsi_frsizey) then
              ridx_it(it) = 2*irsi_frsizey + 1 - ridx_it(it)
          endif
#ifdef MIRROR_THETA
          if(ridx_it(it).gt.irsi_frsizey/2) then
              ridx_it(it) = irsi_frsizey + 1 - ridx_it(it)
          endif
#endif
      enddo
c
c     Set the flag that the reindexation has been performed
c
      ridx_ready = 321
c
      return
      end






c     --------------------------------------------------------------
c               FIND THE MIDPOINT SOURCE BY INTERPOLATION
c
c     This routine is able to find the value of the source term at
c     any given position 'P' at any given angle. It does so by 
c     interpolation. The ir and it are the position of the point '0'
c     (which can be miles away from the point 'P' in principle).
c     The dr and dt contain the offset of 'P' from '0'. Its integer
c     root is the amount of gridpoints away from '0', while its residu
c     is the position within the grid interval. Example: ir=4, dr=3.5
c     means that '0' is at ir=4, so that 'P' is right in the middle
c     between ir=7 and ir=8. 
c
c     ARGUMENTS:
c       inu          = Index of frequency
c       icr          = 1 : Crossing with R=const grid line
c                    = 2 : Crossing with Theta=const grid line
c                    = 0 : Point lies within gridcell, not on grid line
c       ir           = Index of nearest R=const grid line in negative dir
c       it           = Index of nearest Theta=const grid line in negative dir
c       dr           = [0..1] : 0 = at R=R_ir
c                               1 = at R=R_{ir+1}
c       dt           = [0..1] : 0 = at Theta=Theta_ir , 
c                               1 = at Theta=Theta_{ir+1}
c       mu           = Angle mu
c       phi          = Angle phi
c
c     REQUIRES:
c       This routine requires the re-indexation table to have been
c       initialized. This is done by calling make_index(). This table
c       incorporates all mirror symmetries and boundaries in the 
c       spatial coordinate Theta and angular coordinate phi.
c
c     --------------------------------------------------------------
      function find_src_t(inu,icr,ir,it,dr,dt,mu,phi)
      implicit none
      doubleprecision find_src_t,calc_src_pnt
      integer icr,ir,it,inu
      doubleprecision dr,dt,mu,phi
c
#include "common_grid.h"
c
      integer idr,idt,ir0,it0,iymir
      doubleprecision drr,dtr,src,dy
      dimension src(0:3)
      integer indexr,indext
      doubleprecision thetaclip,theta,rclip,r
      dimension thetaclip(0:3),rclip(0:3)
      dimension indexr(0:3),indext(0:3)
c
c     Check the initialization of the reindexation.
c     
      if(ridx_ready.ne.321) then
          write(*,*) 'ERROR, find_src_t: Not initialized ',
     %               'the reindexation yet!'
          stop 13
      endif
c
c     If 1-D then we're done quickly, since it must be on a gridpoint
c
#ifdef RADGRID_ONEDIM
      if(icr.eq.2) then
          write(*,*) 'Inconsistency in 1-D option find_src_t()'
          stop 1
      endif
      if(icr.eq.1) then
          idr   = dr            ! Trick: from dbl to int automatic trunc
          ir0   = ir + idr      ! Note: do not say ir0 = ir + dr        
          find_src_t = calc_src_pnt(inu,ir0,1,mu,phi)
          return
      else
#ifdef SCONEDIM_ADDPOINT
c
c         If icr.eq.0 in 1-D, then follow normal procedure
c     
          if((dt.ne.0.d0).or.(it.ne.1)) then
              write(*,*) 'Internal error with SC 1-D addpoint'
              stop 63453
          endif
#else
c
c         If icr.eq.0 in 1-D, but SCONEDIM_ADDPOINT not set,
c         then error
c
          write(*,*) 'PROBLEM: in 1-D the icr=3 option is not '
          write(*,*) '         yet available for find_src_t()'
          write(*,*) '         If this problem occured while '
          write(*,*) '         rendering spectrum or image in 1-D'
          write(*,*) '         then it is advised to undefine '
          write(*,*) '         LONGCHAR_EXTRA_POINTS'
          stop 1
#endif
      endif
#endif
c
c     For 2-D we proceed doing the more complicated interpolation stuff
c
#ifndef INTERPOL_SRC_2
#define INTERPOL_SRC_WIDE
#endif
#ifdef INTERPOL_SRC_0
#define INTERPOL_SRC_PNT_2
#endif
#ifdef INTERPOL_SRC_1
#define INTERPOL_SRC_PNT_2
#endif
c
c     Find the index of the radial shell from the dr 
c
      idr   = dr                ! Trick: from dbl to int automatic trunc
      ir0   = ir + idr          ! Note: do not say ir0 = ir + dr        
c
c     Find the real dr (the dr from R_1 to R, instead of from R_0 to R)
c
      drr   = abs(dr - idr)
c
c     Now find the index of the R_1. This is always one further
c     in the direction of drr, except when we are at R_out or R_in.
c     If so, I choose the index the same as ir0.
c
      if(dr.gt.0.d0) then
#ifdef INTERPOL_SRC_WIDE
          indexr(0) = ir0
          indexr(1) = ir0 + 1
          indexr(2) = ir0 + 2
          rclip(0)  = rsi_x_c(indexr(0),1)
          rclip(1)  = rsi_x_c(indexr(1),1)
          rclip(2)  = rsi_x_c(indexr(2),1)
          if(indexr(1).gt.irsi_frsizex) then
              indexr(1) = irsi_frsizex
          endif
          if(indexr(2).gt.irsi_frsizex) then
              indexr(2) = irsi_frsizex
          endif
#else
          indexr(0) = ir0 - 1
          indexr(1) = ir0 
          indexr(2) = ir0 + 1
          rclip(0)  = rsi_x_c(indexr(0),1)
          rclip(1)  = rsi_x_c(indexr(1),1)
          rclip(2)  = rsi_x_c(indexr(2),1)
          if(indexr(0).lt.1) then
              indexr(0) = 1
          endif
          if(indexr(2).gt.irsi_frsizex) then
              indexr(2) = irsi_frsizex
          endif
#endif          
      else
#ifdef INTERPOL_SRC_WIDE
          indexr(0) = ir0
          indexr(1) = ir0 - 1
          indexr(2) = ir0 - 2
          rclip(0)  = rsi_x_c(indexr(0),1)
          rclip(1)  = rsi_x_c(indexr(1),1)
          rclip(2)  = rsi_x_c(indexr(2),1)
          if(indexr(1).lt.1) then
              indexr(1) = 1
          endif
          if(indexr(2).lt.1) then
              indexr(2) = 1
          endif
#else
          indexr(0) = ir0 + 1
          indexr(1) = ir0 
          indexr(2) = ir0 - 1
          rclip(0)  = rsi_x_c(indexr(0),1)
          rclip(1)  = rsi_x_c(indexr(1),1)
          rclip(2)  = rsi_x_c(indexr(2),1)
          if(indexr(0).gt.irsi_frsizex) then
              indexr(0) = irsi_frsizex
          endif
          if(indexr(2).lt.1) then
              indexr(2) = 1
          endif
#endif          
      endif
c
c     Reconstruct the R itself from the above
c
#ifdef INTERPOL_SRC_WIDE
      r = rclip(0) + drr * ( rclip(1) - rclip(0) )
#else
      r = rclip(1) + drr * ( rclip(2) - rclip(1) )
#endif
c
c     Only in 2-D:
c
#ifndef RADGRID_ONEDIM
c
c     Find the the index of the Theta_0 from the dt
c
      idt   = dt                ! Trick: from dbl to int automatic trunc
      it0   = it + idt          ! Note: do not say it0 = it + dt        
c
c     Find the real dt (the dt from Theta_1 to Theta, instead
c     of from Theta(it) to Theta)
c
      dtr   = abs(dt - idt)
c
c     Now complete the 3-point Theta index stencil.
c
      if(dt.gt.0.d0) then
#ifdef INTERPOL_SRC_WIDE
          indext(0) = it0 
          indext(1) = it0 + 1
          indext(2) = it0 + 2
#else
          indext(0) = it0 - 1
          indext(1) = it0 
          indext(2) = it0 + 1
#endif
      else            
#ifdef INTERPOL_SRC_WIDE
          indext(0) = it0 
          indext(1) = it0 - 1
          indext(2) = it0 - 2
#else
          indext(0) = it0 + 1
          indext(1) = it0 
          indext(2) = it0 - 1
#endif
      endif
c
c     Find the corresponding values of theta on this stencil, except for
c     the thetaclip(2) because that might go wrong, since the grid rsi_x_c
c     only has one ghost-gridpoints on each border, while indext(2) could
c     be 2 gridpoints off the grid under certain circumstances.
c
      thetaclip(0) = rsi_x_c(indext(0),2)
      thetaclip(1) = rsi_x_c(indext(1),2)
c
c     Find these circumstances, and find the thetaclip(2). 
c
      if(indext(2).lt.0) then
          thetaclip(2) = -rsi_x_c(2,2)
      elseif (indext(2).gt.irsi_frsizey+1) then
          thetaclip(2) = 6.28318530718d0 - rsi_x_c(irsi_frsizey-1,2)
      else
          thetaclip(2) = rsi_x_c(indext(2),2)
      endif      
c
c     Reconstruct the Theta itself from the above
c
#ifdef INTERPOL_SRC_WIDE
      theta = thetaclip(0) + dtr * ( thetaclip(1) - thetaclip(0) )
#else
      theta = thetaclip(1) + dtr * ( thetaclip(2) - thetaclip(1) )
#endif
c
c     Remap the theta indices in case of boundaries crossed, or a 
c     mirror line is crossed.
c
      indext(0)   = ridx_it(indext(0))
      indext(1)   = ridx_it(indext(1))
      indext(2)   = ridx_it(indext(2))
c
#else /* ifndef RADGRID_ONEDIM */
c
c     In case of 1-D, but midpoint interpolation
c
      indext(0)    = 1
      indext(1)    = 1
      thetaclip(0) = rsi_x_c(indext(0),2)
      thetaclip(1) = rsi_x_c(indext(1),2)
      theta        = thetaclip(0)
c
#endif /* ifndef RADGRID_ONEDIM */
c     
c     In case of 0-th order interpolation, do something about 
c     the drr and dtr
c
#ifdef INTERPOL_SRC_0
      if(drr.gt.0.5d0) then 
          drr=1.d0 
      else 
          drr=0.d0 
      endif
      if(dtr.gt.0.5d0) then 
          dtr=1.d0 
      else 
          dtr=0.d0 
      endif
#endif
c
c     Now that we have bracketed the point at which we wish to find
c     the source term, let us make distinction between three types 
c     of interpolation. icr=1 means that the point lies on a R=const
c     grid line, icr=2 means that the point lies on a Theta=const grid
c     line, and icr=0 means that the point really lies somewhere in
c     between both the Theta and R gridlines.
c
#ifdef INTERPOL_SRC_PNT_2
      if(icr.eq.1) then
c
c         The point 'P' lies on an R=const gridline
c
          src(0) = calc_src_pnt(inu,indexr(0),indext(0),mu,phi)
          src(1) = calc_src_pnt(inu,indexr(0),indext(1),mu,phi)
          find_src_t = (1.d0-dtr)*src(0) + dtr*src(1)
c
      elseif(icr.eq.2) then
c
c         The point 'P' lies on an Theta=const gridline
c
          src(0) = calc_src_pnt(inu,indexr(0),indext(0),mu,phi)
          src(1) = calc_src_pnt(inu,indexr(1),indext(0),mu,phi)
          find_src_t = (1.d0-drr)*src(0) + drr*src(1)
c
      else
c
c         The point 'P' lies somewhere in between all gridlines
c
          src(0) = calc_src_pnt(inu,indexr(0),indext(0),mu,phi)
          src(1) = calc_src_pnt(inu,indexr(0),indext(1),mu,phi)
          src(2) = calc_src_pnt(inu,indexr(1),indext(0),mu,phi)
          src(3) = calc_src_pnt(inu,indexr(1),indext(1),mu,phi)
          find_src_t = (1.d0-drr) * ((1.d0-dtr)*src(0) + dtr*src(1)) 
     %                +       drr * ((1.d0-dtr)*src(2) + dtr*src(3)) 
      endif
#else
      if(icr.eq.1) then
c
c         The point 'P' lies on an R=const gridline
c
          src(0) = calc_src_pnt(inu,indexr(0),indext(0),mu,phi)
          src(1) = calc_src_pnt(inu,indexr(0),indext(1),mu,phi)
          src(2) = calc_src_pnt(inu,indexr(0),indext(2),mu,phi)
          call polint3(thetaclip,src,theta,find_src_t)
c
      elseif(icr.eq.2) then
c
c         The point 'P' lies on an Theta=const gridline
c
          src(0) = calc_src_pnt(inu,indexr(0),indext(0),mu,phi)
          src(1) = calc_src_pnt(inu,indexr(1),indext(0),mu,phi)
          src(2) = calc_src_pnt(inu,indexr(2),indext(0),mu,phi)
          call polint3(rclip,src,r,find_src_t)
c
      else
c
c         The point 'P' lies somewhere in between all gridlines
c
#ifdef RADGRID_ONEDIM
c
c         In 1-D this can be done like if it were a Theta=const line
c
          src(0) = calc_src_pnt(inu,indexr(0),indext(0),mu,phi)
          src(1) = calc_src_pnt(inu,indexr(1),indext(0),mu,phi)
          src(2) = calc_src_pnt(inu,indexr(2),indext(0),mu,phi)
          call polint3(rclip,src,r,find_src_t)
#else
          write(*,*) 'ERROR: interpol.F/find_src_t()'
          write(*,*) '       icr=0: This option is not yet ',
     %      'available for 2nd order interpolation in 2-D'
          stop 14
#endif
      endif
c
c     Make sure that the answer is positive definite
c
      find_src_t = 0.5d0 * ( find_src_t + abs(find_src_t) )
#endif
c
      return
      end



c     --------------------------------------------------------------
c               FIND THE MIDPOINT ALPHA BY INTERPOLATION
c
c     This routine is able to find the value of alpha at
c     any given position 'P' at any given angle. See the similar
c     routine find_src_t() for details. 
c     --------------------------------------------------------------
      function find_alp_t(inu,icr,ir,it,dr,dt,mu,phi)
      implicit none
      doubleprecision find_alp_t,calc_alp_pnt
      integer icr,ir,it,inu
      doubleprecision dr,dt,mu,phi
c
#include "common_grid.h"
c
      integer idr,idt,ir0,ir1,it0,it1,iymir
      doubleprecision drr,dtr,alp
      dimension alp(0:3)
c
c     If 1-D then we're done quickly, since it must be on a gridpoint
c
#ifdef RADGRID_ONEDIM
      if(icr.eq.2) then
          write(*,*) 'Inconsistency in 1-D option find_alp_t()'
          stop 1
      endif
      if(icr.eq.1) then
          idr   = dr            ! Trick: from dbl to int automatic trunc
          ir0   = ir + idr      ! Note: do not say ir0 = ir + dr        
          find_alp_t = calc_alp_pnt(inu,ir0,1,mu,phi)
          return
      else
          if((dt.ne.0.d0).or.(it.ne.1)) then
              write(*,*) 'Internal error with SC 1-D addpoint'
              stop 63455
          endif
      endif
#endif
c
c     For 2-D we proceed doing the more complicated interpolation stuff
c
c
c     Find the index of the radial shell from the dr 
c
      idr   = dr                ! Trick: from dbl to int automatic trunc
      ir0   = ir + idr          ! Note: do not say ir0 = ir + dr        
c
c     Find the real dr (the dr from R_1 to R, instead of from R_0 to R)
c
      drr   = abs(dr - idr)
c
c     Now find the index of the R_1. This is always one further
c     in the direction of drr, except when we are at R_out or R_in.
c     If so, I choose the index the same as ir0.
c
      if(dr.gt.0.d0) then
          if(ir0.lt.irsi_frsizex) then 
              ir1 = ir0 + 1
          else
              ir1 = ir0
          endif
      else
          if(ir0.gt.1) then           
              ir1 = ir0 - 1
          else
              ir1 = ir0
          endif
      endif
c
c     Only in 2-D:
c
#ifndef RADGRID_ONEDIM
c
c     Find the the index of the Theta_0 from the dt
c
      idt   = dt                ! Trick: from dbl to int automatic trunc
      it0   = it + idt          ! Note: do not say it0 = it + dt        
c
c     Find the real dt (the dt from Theta_1 to Theta, instead
c     of from Theta(it) to Theta)
c
      dtr   = abs(dt - idt)
c
c     Now find the index of the Theta_1. This is always one further
c     in the direction of dtr, except when we are near the z-axis.
c     If we are near the z-axis, I choose the index the same as it0.
c
      if(dt.gt.0.d0) then
          if(it0.lt.irsi_frsizey) then 
              it1 = it0 + 1
          else
              it1 = it0
          endif
      else
          if(it0.gt.1) then           
              it1 = it0 - 1
          else
              it1 = it0
          endif
      endif
c
c     Now the mirror stuff. Only the Theta mirror needs to be
c     installed for this isotropic source term.
c
#ifdef MIRROR_THETA
      iymir = (irsi_frsizey+1)/2
      if(it0.gt.iymir) then
          it0 = irsi_frsizey - it0 + 1
      endif
      if(it1.gt.iymir) then
          it1 = irsi_frsizey - it1 + 1
      endif
#endif
c
#else /* ifndef RADGRID_ONEDIM */
      it0 = 1
      it1 = 1
      dtr = 0.d0
#endif /* ifndef RADGRID_ONEDIM */
c
c     For 0-th order interpolation we do an if-statement
c
c#######################
c#######################
#ifdef INTERPOL_SRC_0
      if(drr.gt.0.5d0) then 
          drr=1.d0 
      else 
          drr=0.d0 
      endif
      if(dtr.gt.0.5d0) then 
          dtr=1.d0 
      else 
          dtr=0.d0 
      endif
#endif
c
c     Now that we have bracketed the point at which we wish to find
c     the source term, let us make distinction between three types 
c     of interpolation. icr=1 means that the point lies on a R=const
c     grid line, icr=2 means that the point lies on a Theta=const grid
c     line, and icr=0 means that the point really lies somewhere in
c     between both the Theta and R gridlines.
c
      if(icr.eq.1) then
c
c         The point 'P' lies on an R=const gridline
c
          alp(0) = calc_alp_pnt(inu,ir0,it0,mu,phi)
          alp(1) = calc_alp_pnt(inu,ir0,it1,mu,phi)
          find_alp_t = (1.d0-dtr)*alp(0) + dtr*alp(1)
c
      elseif(icr.eq.2) then
c
c         The point 'P' lies on an Theta=const gridline
c
          alp(0) = calc_alp_pnt(inu,ir0,it0,mu,phi)
          alp(1) = calc_alp_pnt(inu,ir1,it0,mu,phi)
          find_alp_t = (1.d0-drr)*alp(0) + drr*alp(1)
c
      else
c
c         The point 'P' lies somewhere in between all gridlines
c
          alp(0) = calc_alp_pnt(inu,ir0,it0,mu,phi)
          alp(1) = calc_alp_pnt(inu,ir0,it1,mu,phi)
          alp(2) = calc_alp_pnt(inu,ir1,it0,mu,phi)
          alp(3) = calc_alp_pnt(inu,ir1,it1,mu,phi)
          find_alp_t = (1.d0-drr) * ((1.d0-dtr)*alp(0) + dtr*alp(1)) 
     %                +       drr * ((1.d0-dtr)*alp(2) + dtr*alp(3)) 
      endif
c
      return
      end



